@@ -449,7 +449,8 @@ public class BrowserTest {
 	
 	String[] operaMini = {
 		"Opera/9.60 (J2ME/MIDP; Opera Mini/4.2.13337/458; U; en) Presto/2.2.0",
-		"Opera/9.80 (J2ME/MIDP; Opera Mini/5.0.16823/1428; U; en) Presto/2.2.0"
+		"Opera/9.80 (J2ME/MIDP; Opera Mini/5.0.16823/1428; U; en) Presto/2.2.0",
+		"Opera/9.80 (iPhone; Opera Mini/7.0.5/37.6116; U; en) Presto/2.12.423 Version/12.16"
 	};
 	
 	String[] operaCoast = {
@@ -499,7 +500,8 @@ public class BrowserTest {
 			"Mozilla/5.0 (compatible; YandexMarket/1.0; +http://yandex.com/bots)",
 			"Mozilla/5.0 (compatible; Ask Jeeves/Teoma; +http://about.ask.com/en/docs/about/webmasters.shtml)",
 			"ThumbSniper (http://thumbsniper.com)",
-			"Phantom.js bot"
+			"Phantom.js bot",
+			"Mozilla/5.0 (iPhone; CPU iPhone OS 8_1 like Mac OS X) AppleWebKit/600.1.4 (KHTML, like Gecko) Version/8.0 Mobile/12B411 Safari/600.1.4 (compatible; YandexMobileBot/3.0; +http://yandex.com/bots)"
 	};
 	
 	String[] mobileBot = {
